/**
 * @file
 * Provides ElevateZoomPlus loader.
 */

(function ($, Drupal, drupalSettings, _db) {

  'use strict';

  var _id = 'elevatezoomplus';
  var _nick = 'ezPlus';
  var _wrapper = _id + '-wrapper';
  var _mounted = _wrapper + '--on';
  var _unload = false;

  /**
   * ElevateZoomPlus public methods.
   *
   * @namespace
   */
  Drupal.elevateZoomPlus = Drupal.elevateZoomPlus || {
    itemSelector: '.' + _id,
    defaults: drupalSettings.elevateZoomPlus || {},
    ez: null
  };

  /**
   * ElevateZoomPlus gallery item functions.
   *
   * @param {HTMLElement} elm
   *   The .elevatezoomplus-wrapper HTML element.
   */
  function doZoomWrapper(elm) {
    var me = Drupal.elevateZoomPlus;
    var options = me.defaults;
    var selector = me.itemSelector;
    var $elm = $(elm);
    var $target = $(selector, elm);
    var $slick = $('.slick--main', elm);
    var $splide = $('.splide--main', elm);
    var $slider = $('> .slick__slider', $slick);
    var dataConfig = $elm.data(_id) || {};
    var hasMedia = $('.media--video', elm).length;
    var isSlick = $slick.length;
    var currSlide = isSlick ? '.slick-current ' + selector : '.slide.is-active ' + selector;

    // Bail out if we have no config available.
    if (!dataConfig) {
      return;
    }

    // Attach our behaviors.
    options = $.extend({}, options, dataConfig);

    if (options.behaviors) {
      check(options.behaviors);
    }

    if (hasMedia && Drupal.blazyBox) {
      Drupal.blazyBox.attach();
    }

    // Integrates with Splide/ Slick with asNavFor.
    if (!$slider.length) {
      $slider = $splide;
    }

    // Sliders with thumbnail navigation.
    if ($slider.length) {
      zoomIn();

      if (isSlick) {
        $slider.on('beforeChange.ez', zoomOut);
        $slider.on('afterChange.ez', zoomIn);
      }
      else {
        var splide = $slider[0].splide;

        if (splide) {
          splide.on('moved.ez', zoomOut);
          splide.on('active.ez', zoomIn);
        }
      }
    }
    else {
      // Sliders without navigation, Blazy Grid, Gridstack, etc.
      $target.ezPlus(options);
      me.ez = $target.data(_nick);
    }

    /**
     * Closes the zoom window, can only exist one at a time.
     */
    function zoomOut() {
      if (me.ez) {
        me.ez.closeAll();
      }
      if ($('.ZoomContainer').length) {
        $('.ZoomContainer').remove();
      }
    }

    /**
     * Zooms in the image, can only exist one at a time.
     */
    function zoomIn() {
      zoomOut();

      // DOM cleanups fix.
      window.setTimeout(function () {
        $target = $(currSlide, $slider);
        $target.ezPlus(options);

        me.ez = $target.data(_nick);
      });
    }

    /**
     * Triggers video zoom on a click event.
     */
    function triggerZoom() {
      var $item = $(this);
      var $parent = $item.parent();
      var media = $parent.data('media') || $item.data('media') || false;
      var video = media && media.type === 'video';

      // Build own video fullscreen as ElevateZoomPlus is not for video.
      if (video && Drupal.blazyBox) {
        Drupal.blazyBox.open($parent.attr('href'));
      }
    }

    $('.media--switch--' + _id, elm).on('click.ez', triggerZoom);
    $elm.addClass(_mounted);
  }

  /**
   * Workarounds for screwed-up context after AJAX contents.
   *
   * @param {array} behaviors
   *   The currently given Drupal.behaviors.
   * @todo remove post Splide and Slick updates.
   */
  function check(behaviors) {
    if (_unload && behaviors) {
      var len = behaviors.length;
      for (var i = 0; i < len; i++) {
        var item = behaviors[i];
        if (Drupal.behaviors[item]) {
          Drupal.behaviors[item].attach(document);
        }
      }
      _unload = false;
    }
  }

  /**
   * Attaches ElevateZoomPlus behavior to HTML element.
   *
   * @type {Drupal~behavior}
   */
  Drupal.behaviors.elevateZoomPlus = {
    attach: function (context) {

      // Context is unreliable with AJAX contents like product variations, etc.
      context = context instanceof HTMLDocument ? context : document;

      // @todo replace with core/once at 2.x when dropping D8.
      var items = context.querySelectorAll('.' + _wrapper + ':not(.' + _mounted + ')');
      if (items.length) {
        _db.once(_db.forEach(items, doZoomWrapper));
      }

    },
    detach: function (context, settings, trigger) {
      _unload = trigger === 'unload';
    }
  };

}(jQuery, Drupal, drupalSettings, dBlazy));
